import { Knex } from 'knex';
import { groups, tasks, steps, images } from '../db';
import { printTask, printStep } from '../printer';

interface Context {
  db: Knex;
}

export const resolvers = {
  Query: {
    groups: async (_: any, __: any, { db }: Context) => {
      return await groups(db).select('*');
    },
    group: async (_: any, { id }: { id: string }, { db }: Context) => {
      return await groups(db).where('id', id).first();
    },
    tasks: async (_: any, { groupId }: { groupId: string }, { db }: Context) => {
      return await tasks(db).where('group_id', groupId).select('*');
    },
    task: async (_: any, { id }: { id: string }, { db }: Context) => {
      return await tasks(db).where('id', id).first();
    },
    steps: async (_: any, { taskId }: { taskId: string }, { db }: Context) => {
      return await steps(db).where('task_id', taskId).orderBy('order').select('*');
    },
    step: async (_: any, { id }: { id: string }, { db }: Context) => {
      return await steps(db).where('id', id).first();
    },
    recentTasks: async (_: any, __: any, { db }: Context) => {
      return await tasks(db)
        .orderBy('last_printed_at', 'desc')
        .limit(10)
        .select('*');
    },
    frequentTasks: async (_: any, __: any, { db }: Context) => {
      return await tasks(db)
        .orderBy('print_count', 'desc')
        .limit(10)
        .select('*');
    },
  },

  Mutation: {
    createGroup: async (_: any, { name, parentId }: { name: string; parentId?: string }, { db }: Context) => {
      const [id] = await groups(db).insert({
        name,
        parent_id: parentId ? parseInt(parentId) : null,
      });
      return await groups(db).where('id', id).first();
    },
    createTask: async (_: any, { title, groupId }: { title: string; groupId: string }, { db }: Context) => {
      const [id] = await tasks(db).insert({
        title,
        group_id: parseInt(groupId),
        print_count: 0,
      });
      return await tasks(db).where('id', id).first();
    },
    createStep: async (
      _: any,
      { title, instructions, taskId, order }: { title: string; instructions: string; taskId: string; order: number },
      { db }: Context
    ) => {
      const [id] = await steps(db).insert({
        title,
        instructions,
        task_id: parseInt(taskId),
        order,
        print_count: 0,
      });
      return await steps(db).where('id', id).first();
    },
    createImage: async (
      _: any,
      { stepId, originalPath, bwPath, order }: { stepId: string; originalPath: string; bwPath: string; order: number },
      { db }: Context
    ) => {
      const [id] = await images(db).insert({
        step_id: parseInt(stepId),
        original_path: originalPath,
        bw_path: bwPath,
        order,
      });
      return await images(db).where('id', id).first();
    },
    printTask: async (_: any, { id }: { id: string }, { db }: Context) => {
      const task = await tasks(db).where('id', id).first();
      if (!task) throw new Error('Task not found');

      await printTask(task, db);

      return await tasks(db)
        .where('id', id)
        .update({
          print_count: task.print_count + 1,
          last_printed_at: new Date().toISOString(),
        })
        .then(() => tasks(db).where('id', id).first());
    },
    printStep: async (_: any, { id }: { id: string }, { db }: Context) => {
      const step = await steps(db).where('id', id).first();
      if (!step) throw new Error('Step not found');

      await printStep(step);

      return await steps(db)
        .where('id', id)
        .update({
          print_count: step.print_count + 1,
          last_printed_at: new Date().toISOString(),
        })
        .then(() => steps(db).where('id', id).first());
    },
  },

  Group: {
    tasks: async (group: { id: number }, _: any, { db }: Context) => {
      return await tasks(db).where('group_id', group.id).select('*');
    },
  },

  Task: {
    group: async (task: { group_id: number }, _: any, { db }: Context) => {
      return await groups(db).where('id', task.group_id).first();
    },
    steps: async (task: { id: number }, _: any, { db }: Context) => {
      return await steps(db).where('task_id', task.id).orderBy('order').select('*');
    },
  },

  Step: {
    task: async (step: { task_id: number }, _: any, { db }: Context) => {
      return await tasks(db).where('id', step.task_id).first();
    },
    images: async (step: { id: number }, _: any, { db }: Context) => {
      return await images(db).where('step_id', step.id).orderBy('order').select('*');
    },
  },
};